pub struct AsyncService<H, R, F>where
H: Fn(R, Arc<Request<()>>) -> F + Send + Sync + 'static,
R: FromRequest,
R::Context: Clone,
R::Future: 'static,
F: Future<Item = Response<Body>, Error = BoxedError> + Send + 'static,{ /* private fields */ }Expand description
Asynchronous hyper service adapter.
This implements hyper::service::Service, decodes incoming requests using
FromRequest and passes the result to a provided handler closure.
Using this type takes a bit of boilerplate away from your app. Specifically, it handles:
- Suppressing the body of the response when the request used
HEAD. - Turning any
hyperdrive::Errorinto a proper HTTP response.
This type stores an async request handler H and the context needed by the
FromRequest implementation. The context is cloned for every request.
Type Parameters
H: The handler closure. Takes aFromRequestimplementorR, and the original request. Returns a future resolving to the response to return to the client. Shared viaArc.R: The request type expected by the handlerH. ImplementsFromRequest.F: TheFuturereturned by the handler closureH.
Examples
use hyperdrive::{FromRequest, service::AsyncService};
use hyper::{Server, Response, Request, Body};
use futures::prelude::*;
use std::sync::Arc;
#[derive(FromRequest)]
enum Route {
#[get("/")]
Index,
}
let service = AsyncService::new(|route: Route, orig: Arc<Request<()>>| {
// The closure is called with the `FromRequest`-implementing type and
// the original request. It has to return any type implementing
// `Future`.
match route {
Route::Index => {
Ok(Response::new(Body::from("Hello World!"))).into_future()
}
}
});
// Create the server future:
let srv = Server::bind(&"127.0.0.1:0".parse().unwrap())
.serve(service);Implementations
Sourceimpl<H, R, F> AsyncService<H, R, F>
impl<H, R, F> AsyncService<H, R, F>
Sourcepub fn new(handler: H) -> Self
pub fn new(handler: H) -> Self
Creates an AsyncService from a handler closure.
This will pass a NoContext to the FromRequest implementation,
which will work fine as long as your type doesn’t require a custom
context. If you need to pass a custom context, refer to
with_context.
Sourceimpl<H, R, F> AsyncService<H, R, F>
impl<H, R, F> AsyncService<H, R, F>
Sourcepub fn with_context(handler: H, context: R::Context) -> Self
pub fn with_context(handler: H, context: R::Context) -> Self
Creates an AsyncService that will call handler to process incoming
requests.
Parameters
handler: The handler closure. This is stored in anArcand is passed every decoded requestR. Returns a futureFresolving to the response to return.context: The context to pass to yourFromRequestimplementor.